import {
  AfterContentInit,
  Component,
  ElementRef,
  OnChanges,
  OnDestroy,
  Output,
  ViewChild,
  SimpleChanges,
  EventEmitter,
  Input
} from '@angular/core';

import { HttpClient } from '@angular/common/http';

/**
* You may include a different variant of BpmnJS:
*
* bpmn-viewer  - displays BPMN diagrams without the ability
*                to navigate them
* bpmn-modeler - bootstraps a full-fledged BPMN editor
*/

import BpmnJS from 'bpmn-js/dist/bpmn-modeler.production.min.js';

@Component({
  selector: 'ngx-threeo-bpmn',
  template: `<div #ref class="diagram-container"></div>`,
  styles: [
    `
    .diagram-container {
      height: 100%;
      width: 100%;
    }
  `
  ]
})
export class NgxThreeoBpmnComponent implements AfterContentInit, OnChanges, OnDestroy {

  private bpmnJS: BpmnJS;

  @ViewChild('ref', { static: true }) private el: ElementRef;
  @Output() private importDone: EventEmitter<any> = new EventEmitter();
  @Output() private createEnded: EventEmitter<any> = new EventEmitter();
  @Output() private connectEnded: EventEmitter<any> = new EventEmitter();
  @Output() private elementClicked: EventEmitter<any> = new EventEmitter();

  @Input() xml: string;

  constructor(private http: HttpClient) {

    this.bpmnJS = new BpmnJS();

    this.bpmnJS.on('import.done', ({ obj, error }: any) => {
      if (!error) {
        this.bpmnJS.get('canvas').zoom('fit-viewport');
        this.importDone.emit(obj);
      }
    });

    this.bpmnJS.on('create.ended', (obj) => {
      if (!obj) return;

      this.createEnded.emit(obj);
    });

    this.bpmnJS.on('connect.ended', (obj) => {
      this.connectEnded.emit(obj);
    });

    this.bpmnJS.on('element.dblclick', (obj) => {
      this.elementClicked.emit(obj);
    });
  }

  ngAfterContentInit(): void {
    this.bpmnJS.attachTo(this.el.nativeElement);
  }

  ngOnChanges(changes: SimpleChanges) {
    // re-import whenever the url changes
    if (changes['xml']) {
      this.importDiagram(changes['xml'].currentValue);
    }
  }

  ngOnDestroy(): void {
    this.bpmnJS.destroy();
  }

  /**
   * Creates a Promise to import the given XML into the current
   * BpmnJS instance, then returns it as an Observable.
   *
   * @see https://github.com/bpmn-io/bpmn-js-callbacks-to-promises#importxml
   */
  private importDiagram(xml: string) {
    if (!xml) {
      this.bpmnJS.get('canvas').zoom('fit-viewport');
      return;
    }

    this.bpmnJS.importXML(xml)
  }

  async salvar() {
    return await this.bpmnJS.saveXML();
  }

  updateLabel(id, name) {
    var elementRegistry = this.bpmnJS.get('elementRegistry');

    var sequenceFlowElement = elementRegistry.get(id);
    const modeling = this.bpmnJS.get('modeling');

    modeling.updateLabel(sequenceFlowElement, name);
  }

  updateProperties(element, properties) {
    var modeling = this.bpmnJS.get('modeling');

    modeling.updateProperties(element, properties);
  }
}
